/*
   author:  Rony G. Flatscher
   date:    2005-04-18/19/20
   changed: 2005-05-03, ---rgf, made sure that exceptions are propagated
            2005-06-05, ---rgf, uppercasing "OOo.cls" to "OOO.CLS", removing
   version: 0.0.4
   name:    OOo.cls
   purpose: ease interfacing with OpenOffice.org, v1.1.x and up
   needs:   BSF4Rexx
*/


   -- if no environment entry, then create the entry with default classes
if .local~hasentry("OOo")=.false then
do
   .local~OOo=.directory~new  -- create a new directory object and store it in .local

   -- preload classes
   l = .list~of(                                            -
      "com.sun.star.beans.PropertyValue"                  , -
      "com.sun.star.comp.helper.Bootstrap"                , -
      "com.sun.star.uno.AnyConverter"                     , -
      "com.sun.star.uno.UnoRuntime"                       , -
      "com.sun.star.lang.Locale"                          , -            -- Ahammer
      "com.sun.star.table.TableSortField"                 , -            -- Ahammer
      "com.sun.star.table.CellRangeAddress"               , -            -- Ahammer
      "com.sun.star.sheet.TableFilterField"                 -            -- Ahammer
      )

   do c over l -- get Java class object and save it in .OOo by className
      call OOo.loadClass c
   end

   .OOo~noProps=.bsf~bsf.createArray(.OOo~propertyValue, 0)   -- create empty array of property-value

   -- list of Interface classes to be loaded at runtime (needs completion)
   l = OOo.getInterfaceList()    -- retuns > 950 OOo-interface classes in a list

   -- save interface class names with which we want to deal
   d=.directory~new
   do i over l
     idx=substr(i, lastpos(".", i)+1)
     d~setentry(i, i)    -- by fully qualified name
     d~setentry(idx, i)  -- by last word
   end
   .OOo~XInterfaces = d  -- save directory
end



::requires BSF.CLS                -- get BSF4Rexx support
::requires OOO_XINTERFACES.REX    -- get the list of all of OOo XInterface classes


      -- load OOo-class and save it in environment
::routine OOo.loadClass public
     parse arg className, idx

     if idx="" then idx=substr(className, lastpos(".", className)+1) -- extract last word to serve as index
     if .OOo~hasentry(idx)=.true then    -- e.g. last word is used as a class in a another package as well
        .error~say("OOo.loadClass:" pp(idx) "already used, in hand:" pp(className) "in .OOo:" pp(.OOo~entry(idx)))

-- say "className="className "idx="idx
     tmpClass=.OOo_proxy~new(.bsf4rexx~class.class~forName(className))  -- wrap it up
     .OOo~setentry(idx, tmpClass)   -- load the given Java class and save it in .OOo
     return tmpClass



      -- connects to given UNO server, returns the XMultiServiceFactory interface object of the remote service manager
::routine OOo.connect public
  parse arg unoURL
      -- no unoURL given, use default
  if unoURL="" then unoURL="uno:socket,host=localhost,port=8100;urp;StarOffice.NamingService"

  context=.OOo~Bootstrap~createInitialComponentContext(.nil)
  uur= context~getServiceManager() ~createInstanceWithContext("com.sun.star.bridge.UnoUrlResolver", context)

  remoteObject        = uur~XUnoUrlResolver ~resolve(unoURL)
  remoteNamingService = remoteObject~XNamingService

  remoteServiceManager=remoteNamingService~getRegisteredObject("StarOffice.ServiceManager")
  remoteMSF           = remoteServiceManager~XMultiServiceFactory
  if .OOo~hasentry("remoteMSF")=.false then .OOo~remoteMSF=remoteMSF -- save remote MultiServiceFactory
  return remoteMSF  -- return the remote XMultiServiceFactory



      -- get and return the XDesktop interface object
::routine OOo.getDesktop
  USE ARG remoteMSF

  if remoteMSF="" | remoteMSF=.nil then remoteMSF=.OOo~remoteMSF -- use the stored remote MultiServiceFactory
  remoteDesktop=remoteMSF~createInstance("com.sun.star.frame.Desktop")
      -- return the XDesktop interface of the remote desktop
  return .OOo~unoRuntime~queryInterface(.OOo~XDesktop, remoteDesktop)



   -- modelled after VB's iif(), some people are so used to it that they must have it at all times
::routine iif public
  if arg(1)=.true then return arg(2)
                  else return arg(3)



   -- a routine for inserting values or formulas into cells
::routine OOo.setCell public
   use arg xSheet, x, y, content

	xSheet~getCellByPosition(x, y) ~setFormula(content)



/* *************************************************************
   Class:       OOo_PROXY
   Purpose:     allows to wrap up Java objects as Object Rexx objects;
                if instances are destroyed, then no derigstration from
                the BSF registry takes place
   restriction: needed to make getting interface objects easier
   ************************************************************* */
::CLASS OOo_PROXY -- subclass BSF PUBLIC

::Method bsfObject ATTRIBUTE

::METHOD init
   expose arg bsfObject -- direct access to BSF object
   use arg bsfObject    -- assign object to attribute
   self~objectName=bsfObject~objectName   -- use same objectname as BSF proxy (= idx into BSF-registry)

::METHOD unknown

   expose arg bsfObject -- direct access to BSF object
   parse arg xName

SIGNAL ON SYNTAX NAME ANY           -- intercept all exceptions
   if xName~left(1)~translate="X" then -- if an interface object is asked for, query and return it
   do
      xClass=.OOo~entry(xName)-- try to get interface class object
      if xClass=.nil then        -- no entry as of yet
      do
         idx=substr(xName, lastpos(".", xName)+1)
         xClass=OOo.loadClass(.ooo~XInterfaces~entry(xName), idx)
      end

         -- query and return the interface object
      return .OOo_proxy~new(.OOo~unoRuntime~queryInterface(xClass, bsfObject))
   end
   else  -- pass to super class (i.e. BSF) to handle it
   do
      if bsfObject~hasmethod(xName) then -- o.k. it's a BSF method (could be ooRexx OBJECT method as well!)
         FORWARD TO (bsfObject) MESSAGE (xName) ARGUMENTS (arg(2))
      else
         FORWARD TO (bsfObject) CONTINUE -- let BSF handle this
      tmpObj=result
      -- if a BSF-object, wrap it up
      if tmpObj~hasmethod("bsf.setFieldValue") then return .OOo_proxy~new(tmpObj)
                                               else return tmpObj
   end

ANY:
  RAISE PROPAGATE    -- raise exception in caller

